import * as ng from 'angular';
import * as _ from 'lodash';
import {findIndexOfItem} from '../../grid-rowSelectionMethod';


export default class SelectionController {

    private gridApi:any;
    private selectedItems:Array<any>;
    private areAllItemsSelected:boolean;
    private uniqueKey:string;
    private itemName;

    static $inject = ['$scope', '$timeout', '$i18next'];

    constructor(private $scope:angular.IScope, private $timeout, private $i18next) {

        //setting item name.
        this.itemName = this.itemName || $i18next.t("nwfjs:items");

        $scope.$watch(()=> this.selectedItems, (newVal, oldVal) => {
            // if all items are selected, we update localStorage with all newIds.
            if (this.areAllItemsSelected) {
                //We clear the localStorage in case any selectedItems are stored in it.
                this.deleteSelectedObjects();
                //Set localStorage with all the fetched IDs.
                this.setSelectedObjects(newVal);
            }

            //if gridApi exists and if allSelectState is set to False
            //This case happens when we cherry pick certain items from selectAll cases.
            //Then we need to make this flag to false, as some items are cherry-picked from selectAll case.
            if (this.gridApi && !this.gridApi.selection.getSelectAllState()) {
                //We make areAllItemsSelected set to false.
                this.areAllItemsSelected = false;
            }

            //we only update localStorage, if newVal or oldVal already exists.
            if ((newVal.length > 0) ||(oldVal.length > 0)) {
                //Updating items in localStorage.
                this.setSelectedObjects(newVal);
            }

        }, true);

        $scope.$on('newDataFetched', (evt, gridData) => {

            //On every new data fetch we check for two things:
            //1) We check if a row's ID of new data exists in selectedItems. If it does exist, we select that row.
            //2) If row's ID does NOT exist in selectedItem, we check if areAllItemSelected flag = true which means we want to select complete set.
            // This means we select all the newly fetched data rows from the response.
            var selectedItems = this.getSelectedObjects();
            if (selectedItems && selectedItems.length > 0) {
                //Looping over new data response.
                gridData.forEach((rowData) => {
                    // Finding the index of new data row from selectedItems.
                    var index;
                    //If we have array of objects stored in selectedItems (we check this by finding out type of first value in selectedItems), we send the entire 'data' object one-by-one into findItemById.
                    //Else, we have non-object type items stored in selectedItems, we send that uniqueKeys key from data object one-by-one into findItemById.
                    if (typeof selectedItems[0] === 'object') {
                        index = findIndexOfItem(selectedItems, rowData, this.uniqueKey);
                    } else {
                        index = findIndexOfItem(selectedItems, rowData[this.uniqueKey], this.uniqueKey);
                    }
                    //1)
                    if (index != -1) {
                        $timeout(()=> {
                            this.gridApi.selection.selectRow(rowData);
                        });
                    }
                    //2)
                    else {
                        if (this.areAllItemsSelected) $timeout(()=> {
                            this.gridApi.selection.selectRow(rowData);
                        });
                    }
                });
            }
        });
    }

    /**removeSelectionOnCurrentPage
     * @ngdoc function
     * @name selectAllOnCurrentPage
     * @description method to select all rows displayed on current page.
     */
    selectAllOnCurrentPage() {

        //setting allObjects to false, in case its true.
        this.areAllItemsSelected = false;

        //selecting all the rows on the current page.
        this.gridApi.selection.selectAllVisibleRows();

        // Store a string representation of object ID(s)
        this.setSelectedObjects(this.selectedItems);
    }

    /**
     * @ngdoc function
     * @name removeSelectionOnCurrentPage
     * @description method to De-Select the selections made on current page.
     */
    deselectAllOnCurrentPage() {
        //setting allObjects to false, in case its true.
        this.areAllItemsSelected = false;

        var selectedItems = this.getSelectedObjects();
        if (selectedItems && selectedItems.length > 0) {
            this.gridApi.grid.options.data.forEach((rowData) => {
                var index;
                if (typeof selectedItems[0] === 'object') {
                    index = findIndexOfItem(selectedItems, rowData, this.uniqueKey);
                } else {
                    index = findIndexOfItem(selectedItems, rowData[this.uniqueKey], this.uniqueKey);
                }
                //If data item ID exist in selectedItems, we delete it from selectedItems.
                if (index !== -1) {
                    selectedItems.splice(index, 1);
                }
            });
            //Updating selectedItems with new selections.
            this.selectedItems = selectedItems;
        }

        //making all the rows de-selected.
        this.$timeout(()=> {
            this.gridApi.selection.clearSelectedRows();
        });

        // If selectedItems exist, we update it, else we remove the empty key from localStorage.
        if (this.selectedItems.length > 0) {
            //Update the selected objects in localStorage.
            this.setSelectedObjects(this.selectedItems);
        }
        else {
            //removing key from localStorage.
            this.deleteSelectedObjects();
        }

    }

    /**
     * @ngdoc function
     * @ngdoc function
     * @name selectCompleteSet
     * @description method to select the complete set of records.
     */
    selectCompleteSet() {
        //setting allObjects to true.
        this.areAllItemsSelected = true;
        //emitting the event, that will be listened in page controller, which in turn fetches all the IDs and pushes the fetched IDs into objectIDs array.
        this.$scope.$emit('fetchAllItems');
        //making all rows display as selected.
        this.gridApi.selection.selectAllVisibleRows();
    }

    /**
     * @ngdoc function
     * @name removeSelections
     * @description method to De-Select the selections made.
     */
    deselectCompleteSet() {
        //setting objectIDs array to zero.
        this.selectedItems.length = 0;
        //making all the rows de-selected.
        this.gridApi.selection.clearSelectedRows();
        //setting allObjects to false, in case its true.
        this.areAllItemsSelected = false;
        //removing selected objects from localStorage.
        this.deleteSelectedObjects();
    }

    setSelectedObjects(selectedItems) {
        // Store a string representation of object ID(s)
        window.localStorage && window.localStorage.setItem('selectedItems', JSON.stringify(selectedItems));
    }

    getSelectedObjects() {
        // Returns an array of object ids
        return window.localStorage ? JSON.parse(window.localStorage.getItem('selectedItems')) : [];
    }

    deleteSelectedObjects() {
        window.localStorage.removeItem('selectedItems');
    }


}
