import * as angular from 'angular';

export default {
    enableFiltering: true,
    paginationPageSize: 20
}

/**
 * @ngdoc function
 * @name gridClientPagination
 * @description method to implement client side PAGINATION for grid.
 */
export function gridClientPagination(oldOffset, newOffset, gridAPI) {
    var totalPages = gridAPI.pagination.getTotalPages();
    if (newOffset === 0) {
        gridAPI.pagination.seek(1);
    }
    else if (newOffset === (totalPages - 1) * gridAPI.grid.options.paginationPageSize) {
        gridAPI.pagination.seek(totalPages);
    }
    else if (newOffset < oldOffset) {
        gridAPI.pagination.previousPage();
    }
    else if (newOffset > oldOffset) {
        gridAPI.pagination.nextPage();
    }
}

/**
 * @ngdoc function
 * @name clientFilter
 * @description method to implement client side FILTERING for grid.
 */
export function gridClientFilter(gridAPI, uiGridConstants, rowSearcher, filter) {

    this.conditionObject = {
        'gt': uiGridConstants.filter.GREATER_THAN,
        'lt': uiGridConstants.filter.LESS_THAN,
        'ct': uiGridConstants.filter.CONTAINS,
        'sw': uiGridConstants.filter.STARTS_WITH,
        'is': uiGridConstants.filter.EXACT,
        'nis': uiGridConstants.filter.NOT_EQUAL
    };

    //Finding all filters to be applied.
    var appliedFilters:Array<FilterDef> = [];
    for (var i = 0; i < filter.length; i++) {
        //Splitting each filter values.
        var filterParam = filter[i].split(',');
        //creating new filter object.
        var filterName = filterParam[0], operator = filterParam[1], filterValue = filterParam[2];
        appliedFilters.push({
            columnName: filterName,
            //if operator is '<' OR '>', run a parseInt of number entered, else just set value as-is
            term: ((operator === 'gt') || (operator === 'lt')) ? parseInt(filterValue) : filterValue,
            condition: this.conditionObject[operator]
        });
    }

    // Finding reference to the grid columns.
    var cols = gridAPI.grid.columns;
    var filterSet = false;
    //Applying the filters to the columns.
    for (var c = 0; c < cols.length; c++) {
        //set flag to false initially.
        filterSet = false;
        //we set the filters to empty initially.
        cols[c].filters = [];
        //finding the column (by matching to columnName) that has the filter applied from 'appliedFilters' list;
        var columnWithFilter:Array<FilterDef> = appliedFilters.filter(function (eachFilter) {
            return eachFilter.columnName === cols[c].field;
        });
        if (columnWithFilter && columnWithFilter.length === 1) {
            //we add the filter to filter property of column.
            cols[c].filters.push({term: columnWithFilter[0].term, condition: columnWithFilter[0].condition});
        } else {
            // we set filters to empty to remove any existing filters.
            cols[c].filters = [];
        }
    }

    // Searching across all the rows and columns of the grid for applied filters.
    rowSearcher.search(gridAPI.grid, gridAPI.grid.rows, gridAPI.grid.columns);

    //refresh the grid.
    gridAPI.grid.refresh();

    // Returning the count of totalItems.
    return gridAPI.grid.options.totalItems;
}

/**
 * @ngdoc function
 * @name clientSearch
 * @description method to implement client side SEARCH for grid.
 */
export function gridClientSearch(renderableRows, columns, newQuery) {

    //getting the regex of the query parameter.
    var matcher = new RegExp(newQuery);
    // if query is entered.
    if (newQuery) {
        // looping over each row.
        renderableRows.forEach(function (row) {
            var match = false;
            //starting from first column since we have selection as col=0.
            for (var col = 1; col < columns.length; col++) {
                if (typeof(row.entity[columns[col].field]) === 'string' && row.entity[columns[col].field].match(matcher)) {
                    match = true;
                }
            }
            if (!match) {
                // setting row visibility to false if it does not matches the query string.
                row.visible = false;
            }
        });
    }

    //returning the count of visible rows ie. returning the rows that satisfy the query string critieria.
    return renderableRows.filter(function (row) {
        return row.visible;
    }).length;
}

/**
 * @ngdoc function
 * @name set grid Column type
 * @description method to set type property on grid columns 'columnDef' that aids in sorting.
 */
export function gridSetColumnType(columnType){
    var type='string';
    switch(columnType){
        case 'metric':case 'number':
        type = 'number';
        break;
        default:
            type='string';
            break;
    }
    return type;
}