import * as angular from 'angular';
import * as DateUtils from './DateUtils';
import * as LocaleUtils from './LocaleUtils';
import * as MathUtils from './MathUtils';
import * as StringUtils from './StringUtils';

export default class FiltersModule {

    static NAME: string = 'NwfJsFiltersModule';
    private filtersModule: angular.IModule;

    constructor(){
        this.filtersModule = angular.module(FiltersModule.NAME, []);

        /**
         * Round and display a value as defined in the MathUtils.round function
         * Add units to the value if defined and set
         */
        this.filtersModule.filter('round', function() {
            return function(value, units, isPercent, statType) {
                var text = MathUtils.round(value, isPercent, statType);
                if(text !== null){ // add units only if value is not null
                    if(units) { // add units only if units is not undefined and not null
                        text += (isPercent ? '' : ' ' ) + units;
                    }
                }
                return text;
            };
        });

        /**
         * Round and format number as locale and display a value as defined in the MathUtils.round function
         * Add units to the value if defined and set
         */
        this.filtersModule.filter('RoundWithLocale', function() {
            return function(value, units, isPercent, statType) {

                var text = MathUtils.round(value, isPercent, statType);
                if(text){ //parsing through number locale if value is truthy.
                    text = LocaleUtils.numberLocale(text);
                    if(units) { // add units only if units is not undefined and not null
                        text += (isPercent ? '' : ' ' ) + units;
                    }
                }
                return text;
            };
        });

        /**
         * Used to display ellipsis for a null value
         */
        this.filtersModule.filter('Ellipsize', function() {
            return function(value) {
                return (value || String.fromCharCode(8230));
            };
        });

        /**
         * Used to display N/A for a null value
         */
        this.filtersModule.filter('NotAvailable', function() {
            return function(value) {
                return (value || 'N/A');
            };
        });

        /**
         * Used to display 'NotSupported' for non-supported stat types. Depends on Cluster Version.
         */
        this.filtersModule.filter('NotSupported', function() {
            return function(value, statType, unsupportedStatisticTypes) {
                if(unsupportedStatisticTypes && unsupportedStatisticTypes.length > 0 && unsupportedStatisticTypes.indexOf(statType) >= 0){
                    return 'Not Supported';
                }
                else{
                    return value;
                }
            };
        });

        /**
         * Filter to print array of policy object into comma-separated policy names.
         *
         * e.g: [{id:1,label:pol1},{id:1,label:pol2},{id:1,label:pol3}] returns pol1,pol2,pol3
         */
        this.filtersModule.filter('policyFilter',function(){
            return function(myArray) {
                var returnArray = [];
                if(myArray && myArray.length > 0){
                    myArray.forEach(function(value){
                        returnArray.push(value.label);
                    });
                    return returnArray.join(', ');
                }
            };
        });

        /**
         * Convert text to title case
         *
         * Example:
         * volume -> Volume
         * ulhas pai -> Ulhas Pai
         *
         */
        this.filtersModule.filter('TitleCase',function(){
            return function(str) {
                return str.replace(/\w\S*/g, function(txt) {
                    return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase();
                });
            };
        });

        this.filtersModule.filter('ObjectType', function(){
            return function(objectType) {
                switch(objectType) {
                    case "cluster" : return "Cluster";
                    case "node" : return "Node";
                    case "aggregate" : return "Aggregate";
                    case "svm" : return "SVM";
                    case "volume" : return "Volume";
                    case "lun" : return "LUN";
                    case "port" : return "Port";
                    case "lif" : return "LIF";
                    default : return objectType;
                }
            };
        });

        this.filtersModule.filter('CompositeString', function(){
            return function(objectSubType) {
                switch(objectSubType) {
                    case "FLEXGROUP" : return "FlexGroup";
                    case "CONSTITUENT" : return "Constituent Volume";
                    default : return objectSubType;
                }
            };
        });

        /**
         * Converts Regular nouns to plural forms
         *
         * Example:
         * volume -> volumes
         * Policy -> policies
         * Box -> boxes
         *
         */
        this.filtersModule.filter('pluralize', function(){
            return function(string) {
                return StringUtils.pluralize(string);
            };
        });


        /**
         * Converts milliseconds to hours and minutes.
         * @param milliseconds
         */
        this.filtersModule.filter('getDisplayTextForMinutes', function(){
            return function(mins) {
                if(mins !== null){
                    return DateUtils.getDisplayTextForMinutes(mins);
                }

            };
        });

        /**
         * Localizes the number to be displayed.
         * E.g 123456 might be displayed as 123,456
         *
         * @param number - the number to be localized
         */
        this.filtersModule.filter('NumberLocale', function(){
            return function(number) {
                return LocaleUtils.numberLocale(number);
            };
        });
    }
}